@@ -98,33 +98,21 @@ def list_files(
     recursive: bool = True,
     include_hidden: bool = False,
     excluded_patterns: Optional[list[str]] = None,
-) -> str:
+) -> dict[str, list[str]]:
     """
-    Lists the files and directories within a specified path.
+    Lists files and directories within a specified path, providing a map of the filesystem.
 
-    This is a fundamental tool for exploring the file system. Use it to
-    discover the structure of a directory, find specific files, or get a
-    general overview of the project layout before performing other operations.
+    Use this tool to explore and understand the directory structure of the project. It's essential for finding files before reading or modifying them. If you receive an error about a file not being found, use this tool to verify the correct path.
 
     Args:
-        path (str, optional): The directory path to list. Defaults to the
-            current directory (".").
-        recursive (bool, optional): If True, lists files and directories
-            recursively. If False, lists only the top-level contents.
-            Defaults to True.
-        include_hidden (bool, optional): If True, includes hidden files and
-            directories (those starting with a dot). Defaults to False.
-        excluded_patterns (list[str], optional): A list of glob patterns to
-            exclude from the listing. This is useful for ignoring irrelevant
-            files like build artifacts or virtual environments. Defaults to a
-            standard list of common exclusion patterns.
+        path (str, optional): The directory path to list, preferably an absolute path. Defaults to the current directory (".").
+        recursive (bool, optional): If True, lists contents recursively. Defaults to True.
+        include_hidden (bool, optional): If True, includes hidden files (e.g., .gitignore). Defaults to False.
+        excluded_patterns (list[str], optional): A list of glob patterns to ignore (e.g., ["*.log", "node_modules/"]). Defaults to a standard list of ignores.
 
     Returns:
-        str: A JSON string containing a list of file and directory paths
-            relative to the input path.
-            Example: '{"files": ["src/main.py", "README.md"]}'
-    Raises:
-        FileNotFoundError: If the specified path does not exist.
+        dict[str, list[str]]: A dictionary with a single key "files" containing a sorted list of file and directory paths relative to the input path.
+        Example: {"files": ["README.md", "src/main.py", "src/utils/helpers.py"]}
     """
     all_files: list[str] = []
     abs_path = os.path.abspath(os.path.expanduser(path))
@@ -173,14 +161,14 @@ def list_files(
         # Return paths relative to the original path requested
         try:
             rel_files = [os.path.relpath(f, abs_path) for f in all_files]
-            return json.dumps({"files": sorted(rel_files)})
+            return {"files": sorted(rel_files)}
         except (
             ValueError
         ) as e:  # Handle case where path is '.' and abs_path is CWD root
             if "path is on mount '" in str(e) and "' which is not on mount '" in str(e):
                 # If paths are on different mounts, just use absolute paths
                 rel_files = all_files
-                return json.dumps({"files": sorted(rel_files)})
+                return {"files": sorted(rel_files)}
             raise
     except (OSError, IOError) as e:
         raise OSError(f"Error listing files in {path}: {e}")
@@ -221,37 +209,26 @@ def read_from_file(
     path: str,
     start_line: Optional[int] = None,
     end_line: Optional[int] = None,
-) -> str:
+) -> dict[str, Any]:
     """
-        Reads the content of a file, optionally from a specific start line to an
-        end line.
+    Reads the content of a single file.
 
-        This tool is essential for inspecting file contents. It can read both text
-        and PDF files. The returned content is prefixed with line numbers, which is
-        crucial for providing context when you need to modify the file later with
-        the `apply_diff` tool.
+    Use this tool to inspect the contents of a specific file. You can read the entire file or specify a range of lines. The content returned will include line numbers, which are useful for other tools like `replace_in_file`.
 
-        Use this tool to:
-        - Examine the source code of a file.
-        - Read configuration files.
-        - Check the contents of a document.
+    Args:
+        path (str): The absolute path to the file to read.
+        start_line (int, optional): The 1-based line number to start reading from. Defaults to the beginning of the file.
+        end_line (int, optional): The 1-based line number to stop reading at (inclusive). Defaults to the end of the file.
 
-        Args:
-            path (str): The path to the file to read.
-            start_line (int, optional): The 1-based line number to start reading
-                from. If omitted, reading starts from the beginning of the file.
-            end_line (int, optional): The 1-based line number to stop reading at
-                (inclusive). If omitted, reads to the end of the file.
-
-        Returns:
-            str: A JSON object containing the file path, the requested content
-                with line numbers, the start and end lines, and the total number
-                of lines in the file.
-                Example: '{"path": "src/main.py", "content": "1| import os\n2|
-    3| print(\"Hello, World!\")", "start_line": 1, "end_line": 3,
-                "total_lines": 3}'
-        Raises:
-            FileNotFoundError: If the specified file does not exist.
+    Returns:
+        dict[str, Any]: A dictionary containing the file path and its content with line numbers.
+        Example: {
+            "path": "src/main.py",
+            "content": "1| import os\n2|\n3| print(\"Hello, World!\")",
+            "start_line": 1,
+            "end_line": 3,
+            "total_lines": 3
+        }
     """
 
     abs_path = os.path.abspath(os.path.expanduser(path))
@@ -275,15 +252,13 @@ def read_from_file(
         # Select the lines for the result
         selected_lines = lines[start_idx:end_idx]
         content_result = "\n".join(selected_lines)
-        return json.dumps(
-            {
-                "path": path,
-                "content": content_result,
-                "start_line": start_idx + 1,  # Convert back to 1-based for output
-                "end_line": end_idx,  # end_idx is already exclusive upper bound
-                "total_lines": total_lines,
-            }
-        )
+        return {
+            "path": path,
+            "content": content_result,
+            "start_line": start_idx + 1,  # Convert back to 1-based for output
+            "end_line": end_idx,  # end_idx is already exclusive upper bound
+            "total_lines": total_lines,
+        }
     except (OSError, IOError) as e:
         raise OSError(f"Error reading file {path}: {e}")
     except Exception as e:
@@ -295,22 +270,17 @@ def write_to_file(
     content: str,
 ) -> str:
     """
-    Writes content to a file, completely overwriting it if it exists or
-    creating it if it doesn't.
+    Writes content to a file, creating it or completely overwriting it.
 
-    Use this tool to create new files or to replace the entire content of
-    existing files. This is a destructive operation, so be certain of your
-    actions. Always read the file first to understand its contents before
-    overwriting it, unless you are creating a new file.
+    Use this tool to create a new file or replace an existing file's entire content.
+    **WARNING:** This is a destructive operation and will overwrite the file if it exists. Use `replace_in_file` for safer, targeted changes.
 
     Args:
-        path (str): The path to the file to write to.
-        content (str): The full, complete content to be written to the file.
-            Do not use partial content or omit any lines.
+        path (str): The absolute path of the file to write to.
+        content (str): The full content to be written to the file.
 
     Returns:
-        str: A JSON object indicating success or failure.
-             Example: '{"success": true, "path": "new_file.txt"}'
+        str: A confirmation message indicating the file was written successfully.
     """
     try:
         abs_path = os.path.abspath(os.path.expanduser(path))
@@ -319,8 +289,7 @@ def write_to_file(
         if directory and not os.path.exists(directory):
             os.makedirs(directory, exist_ok=True)
         write_file(abs_path, content)
-        result_data = {"success": True, "path": path}
-        return json.dumps(result_data)
+        return f"Successfully wrote to file: {path}"
     except (OSError, IOError) as e:
         raise OSError(f"Error writing file {path}: {e}")
     except Exception as e:
@@ -332,7 +301,7 @@ def search_files(
     regex: str,
     file_pattern: Optional[str] = None,
     include_hidden: bool = True,
-) -> str:
+) -> dict[str, Any]:
     """
     Searches for a regular expression (regex) pattern within files in a
     specified directory.
@@ -352,7 +321,7 @@ def search_files(
             hidden files and directories. Defaults to True.
 
     Returns:
-        str: A JSON object containing a summary of the search and a list of
+        dict[str, Any]: A dictionary containing a summary of the search and a list of
             results. Each result includes the file path and a list of matches,
             with each match showing the line number, line content, and a few
             lines of context from before and after the match.
@@ -404,9 +373,7 @@ def search_files(
                 f"Found {match_count} matches in {file_match_count} files "
                 f"(searched {searched_file_count} files)."
             )
-        return json.dumps(
-            search_results
-        )  # No need for pretty printing for LLM consumption
+        return search_results
     except (OSError, IOError) as e:
         raise OSError(f"Error searching files in {path}: {e}")
     except Exception as e:
@@ -452,25 +419,15 @@ def replace_in_file(
     """
     Replaces the first occurrence of a string in a file.
 
-    This tool is for making targeted modifications to a file. It is a
-    single-step operation that is generally safer and more ergonomic than
-    `write_to_file` for small changes.
-
-    To ensure the replacement is applied correctly and to avoid ambiguity, the
-    `old_string` parameter should be a unique, multi-line string that includes
-    context from before and after the code you want to change.
+    This tool is for making targeted modifications to a file. It is safer than `write_to_file` for small changes. First, use `read_from_file` to get the exact text block you want to change. Then, use that block as the `old_string`.
 
     Args:
-        path (str): The path of the file to modify.
-        old_string (str): The exact, verbatim string to search for and replace.
-            This should be a unique, multi-line block of text.
+        path (str): The absolute path of the file to modify.
+        old_string (str): The exact, verbatim string to replace. This should be a unique, multi-line block of text copied from the file.
         new_string (str): The new string that will replace the `old_string`.
 
     Returns:
-        str: A JSON object indicating the success or failure of the operation.
-    Raises:
-        FileNotFoundError: If the specified file does not exist.
-        ValueError: If the `old_string` is not found in the file.
+        str: A confirmation message on success.
     """
     abs_path = os.path.abspath(os.path.expanduser(path))
     if not os.path.exists(abs_path):
@@ -481,7 +438,7 @@ def replace_in_file(
             raise ValueError(f"old_string not found in file: {path}")
         new_content = content.replace(old_string, new_string, 1)
         write_file(abs_path, new_content)
-        return json.dumps({"success": True, "path": path})
+        return f"Successfully replaced content in {path}"
     except ValueError as e:
         raise e
     except (OSError, IOError) as e:
@@ -492,9 +449,9 @@ def replace_in_file(
 
 async def analyze_file(
     ctx: AnyContext, path: str, query: str, token_limit: int | None = None
-) -> str:
+) -> dict[str, Any]:
     """
-    Performs a deep, goal-oriented analysis of a single file using a sub-agent.
+    Performs a high level, goal-oriented analysis of a single file using a sub-agent.
 
     This tool is ideal for complex questions about a single file that go beyond
     simple reading or searching. It uses a specialized sub-agent to analyze the
@@ -525,7 +482,7 @@ async def analyze_file(
             content to be passed to the analysis sub-agent.
 
     Returns:
-        str: A detailed, markdown-formatted analysis of the file, tailored to
+        dict[str, Any]: A detailed, markdown-formatted analysis of the file, tailored to
             the specified query.
     Raises:
         FileNotFoundError: If the specified file does not exist.
@@ -543,13 +500,16 @@ async def analyze_file(
         tools=[read_from_file, search_files],
     )
     payload = json.dumps(
-        {"instruction": query, "file_path": abs_path, "file_content": file_content}
+        {
+            "instruction": query,
+            "file_path": abs_path,
+            "file_content": llm_rate_limitter.clip_prompt(file_content, token_limit),
+        }
     )
-    clipped_payload = llm_rate_limitter.clip_prompt(payload, token_limit)
-    return await _analyze_file(ctx, clipped_payload)
+    return await _analyze_file(ctx, payload)
 
 
-def read_many_files(paths: list[str]) -> str:
+def read_many_files(paths: list[str]) -> dict[str, str]:
     """
     Reads and returns the full content of multiple files at once.
 
@@ -558,17 +518,43 @@ def read_many_files(paths: list[str]) -> str:
     project relate to each other, or when you need to inspect a set of related
     configuration or source code files.
 
+    The content for each file in the returned dictionary is **always** prefixed
+    with line numbers (`line_number | `) for each line. These prefixes are added
+    by the tool for internal processing and to provide contextual information to
+    the AI agent (e.g., for accurately applying diffs or for referencing specific
+    lines). They are **not** part of the actual file content.
+
+    When presenting the file content to a user, the AI agent **MUST** omit these
+    line number prefixes by default for better readability. They should only
+    be included if explicitly requested by the user or if the request inherently
+    requires line number context (e.g., 'show me lines 10-20').
+
     Args:
         paths (list[str]): A list of paths to the files you want to read. It is
             crucial to provide accurate paths. Use the `list_files` tool first
             if you are unsure about the exact file locations.
 
     Returns:
-        str: A JSON object where keys are the file paths and values are their
-            corresponding contents, prefixed with line numbers. If a file
-            cannot be read, its value will be an error message.
-            Example: '{"results": {"src/api.py": "1| import ...",
-            "config.yaml": "1| key: value"}}'
+        dict[str, str]:  a dictionary where keys are the file paths and values
+            are their corresponding contents (always with line numbers).
+            If a file cannot be read, its value will be an error message.
+            Example:
+            ```
+            {
+                "src/api.py": "1| import os\n2|\n3| print("Hello, World!")",
+                "config.yaml": "1| key: value"
+            }
+            ```
+            If the file 'src/api.py' contains:
+            ```
+            import os
+
+            print("Hello, World!")
+            ```
+            The content for 'src/api.py' in the returned dictionary will be:
+            `"1| import os\n2|\n3| print("Hello, World!")"`
+            **IMPORTANT:** The "1|", "2|", "3|" prefixes are NOT part of the
+            original file. They are for internal agent use.
     """
     results = {}
     for path in paths:
@@ -580,10 +566,10 @@ def read_many_files(paths: list[str]) -> str:
             results[path] = content
         except Exception as e:
             results[path] = f"Error reading file: {e}"
-    return json.dumps({"results": results})
+    return results
 
 
-def write_many_files(files: list[FileToWrite]) -> str:
+def write_many_files(files: list[FileToWrite]) -> dict[str, Any]:
     """
     Writes content to multiple files in a single, atomic operation.
 
@@ -601,20 +587,18 @@ def write_many_files(files: list[FileToWrite]) -> str:
             containing a 'path' and the complete 'content'.
 
     Returns:
-        str: A JSON object summarizing the operation, listing successfully
+        dict[str, Any]: A dictionary summarizing the operation, listing successfully
             written files and any files that failed, along with corresponding
             error messages.
-            Example: '{"success": ["file1.py", "file2.txt"], "errors": {}}'
+            Example: {"success": ["file1.py", "file2.txt"], "errors": {}}
     """
     success = []
     errors = {}
     # 4. Access the data using dictionary key-lookup syntax.
     for file in files:
+        path = file["path"]
+        content = file["content"]
         try:
-            # Use file['path'] and file['content'] instead of file.path
-            path = file["path"]
-            content = file["content"]
-
             abs_path = os.path.abspath(os.path.expanduser(path))
             directory = os.path.dirname(abs_path)
             if directory and not os.path.exists(directory):
@@ -623,4 +607,4 @@ def write_many_files(files: list[FileToWrite]) -> str:
             success.append(path)
         except Exception as e:
             errors[path] = f"Error writing file: {e}"
-    return json.dumps({"success": success, "errors": errors})
+    return {"success": success, "errors": errors}
