@@ -22,7 +22,7 @@ setup_ubuntu = setup_group.add_task(
         description="🐧 Setup ubuntu",
         cmd=[
             "sudo apt install -y \\",
-            "build-essential python3-distutils libssl-dev zlib1g-dev \\"
+            "build-essential libssl-dev zlib1g-dev \\"
             "libbz2-dev libreadline-dev libsqlite3-dev libpq-dev python3-dev \\",
             "llvm libncurses5-dev libncursesw5-dev xz-utils tk-dev libffi-dev \\",
             "liblzma-dev python3-openssl libblas-dev liblapack-dev rustc \\",
