@@ -244,6 +244,21 @@ class Config:
         value = self._getenv("LLM_API_KEY")
         return None if value == "" else value
 
+    @property
+    def LLM_MODEL_SMALL(self) -> str | None:
+        value = self._getenv("LLM_MODEL_SMALL")
+        return None if value == "" else value
+
+    @property
+    def LLM_BASE_URL_SMALL(self) -> str | None:
+        value = self._getenv("LLM_BASE_URL_SMALL")
+        return None if value == "" else value
+
+    @property
+    def LLM_API_KEY_SMALL(self) -> str | None:
+        value = self._getenv("LLM_API_KEY_SMALL")
+        return None if value == "" else value
+
     @property
     def LLM_SYSTEM_PROMPT(self) -> str | None:
         value = self._getenv("LLM_SYSTEM_PROMPT")
@@ -298,14 +313,23 @@ class Config:
         """Maximum number of tokens allowed per individual LLM request."""
         return int(self._getenv("LLM_MAX_TOKENS_PER_REQUEST", "100000"))
 
+    @property
+    def LLM_MAX_TOKENS_PER_TOOL_CALL_RESULT(self) -> int:
+        """Maximum number of tokens allowed per tool call result."""
+        return int(self._getenv("LLM_MAX_TOKENS_PER_TOOL_CALL_RESULT", "75000"))
+
     @property
     def LLM_THROTTLE_SLEEP(self) -> float:
         """Number of seconds to sleep when throttling is required."""
         return float(self._getenv("LLM_THROTTLE_SLEEP", "1.0"))
 
     @property
-    def LLM_YOLO_MODE(self) -> bool:
-        return to_boolean(self._getenv("LLM_YOLO_MODE", "false"))
+    def LLM_YOLO_MODE(self) -> bool | list[str]:
+        str_val = self._getenv("LLM_YOLO_MODE", "false")
+        try:
+            return to_boolean(str_val)
+        except Exception:
+            return [val.strip() for val in str_val.split(",") if val.strip() != ""]
 
     @property
     def LLM_SUMMARIZE_HISTORY(self) -> bool:
@@ -379,14 +403,6 @@ class Config:
     def LLM_ALLOW_SEARCH_INTERNET(self) -> bool:
         return to_boolean(self._getenv("LLM_ALLOW_SEARCH_INTERNET", "1"))
 
-    @property
-    def LLM_ALLOW_SEARCH_ARXIV(self) -> bool:
-        return to_boolean(self._getenv("LLM_ALLOW_SEARCH_ARXIV", "1"))
-
-    @property
-    def LLM_ALLOW_SEARCH_WIKIPEDIA(self) -> bool:
-        return to_boolean(self._getenv("LLM_ALLOW_SEARCH_WIKIPEDIA", "1"))
-
     @property
     def LLM_ALLOW_GET_CURRENT_LOCATION(self) -> bool:
         return to_boolean(self._getenv("LLM_ALLOW_GET_CURRENT_LOCATION", "1"))
@@ -421,10 +437,23 @@ class Config:
     def RAG_MAX_RESULT_COUNT(self) -> int:
         return int(self._getenv("RAG_MAX_RESULT_COUNT", "5"))
 
+    @property
+    def SEARCH_INTERNET_METHOD(self) -> str:
+        """Either serpapi or searxng"""
+        return self._getenv("SEARCH_INTERNET_METHOD", "serpapi")
+
     @property
     def SERPAPI_KEY(self) -> str:
         return os.getenv("SERPAPI_KEY", "")
 
+    @property
+    def SEARXNG_PORT(self) -> int:
+        return int(self._getenv("SEARXNG_PORT", "8080"))
+
+    @property
+    def SEARXNG_BASE_URL(self) -> str:
+        return self._getenv("SEARXNG_BASE_URL", f"http://localhost:{self.SEARXNG_PORT}")
+
     @property
     def BANNER(self) -> str:
         return fstring_format(
