@@ -58,11 +58,15 @@ class BaseInput(AnyInput):
         return f'<input name="{name}" placeholder="{description}" value="{default}" />'
 
     def update_shared_context(
-        self, shared_ctx: AnySharedContext, str_value: str | None = None
+        self,
+        shared_ctx: AnySharedContext,
+        str_value: str | None = None,
+        value: Any = None,
     ):
-        if str_value is None:
-            str_value = self.get_default_str(shared_ctx)
-        value = self._parse_str_value(str_value)
+        if value is None:
+            if str_value is None:
+                str_value = self.get_default_str(shared_ctx)
+            value = self._parse_str_value(str_value)
         if self.name in shared_ctx.input:
             raise ValueError(f"Input already defined in the context: {self.name}")
         shared_ctx.input[self.name] = value
@@ -91,12 +95,20 @@ class BaseInput(AnyInput):
         default_str = self.get_default_str(shared_ctx)
         if default_str != "":
             prompt_message = f"{prompt_message} [{default_str}]"
-        print(f"{prompt_message}: ", end="")
-        value = input()
+        value = self._read_line(shared_ctx, prompt_message)
         if value.strip() == "":
             value = default_str
         return value
 
+    def _read_line(self, shared_ctx: AnySharedContext, prompt_message: str) -> str:
+        if not shared_ctx.is_tty:
+            print(f"{prompt_message}: ", end="")
+            return input()
+        from prompt_toolkit import PromptSession
+
+        reader = PromptSession()
+        return reader.prompt(f"{prompt_message}: ")
+
     def get_default_str(self, shared_ctx: AnySharedContext) -> str:
         """Get default value as str"""
         default_value = get_attr(
