@@ -1,5 +1,7 @@
+import inspect
 import json
 from collections.abc import Callable
+from dataclasses import dataclass
 from typing import TYPE_CHECKING, Any
 
 from zrb.config.llm_rate_limitter import LLMRateLimiter, llm_rate_limitter
@@ -16,6 +18,7 @@ if TYPE_CHECKING:
     from pydantic_ai.agent import AgentRun
     from pydantic_ai.messages import UserContent
     from pydantic_ai.models import Model
+    from pydantic_ai.output import OutputDataT, OutputSpec
     from pydantic_ai.settings import ModelSettings
     from pydantic_ai.toolsets import AbstractToolset
 
@@ -25,16 +28,48 @@ if TYPE_CHECKING:
 def create_agent_instance(
     ctx: AnyContext,
     model: "str | Model",
+    output_type: "OutputSpec[OutputDataT]" = str,
     system_prompt: str = "",
     model_settings: "ModelSettings | None" = None,
     tools: "list[ToolOrCallable]" = [],
-    toolsets: list["AbstractToolset[Agent]"] = [],
+    toolsets: list["AbstractToolset[None]"] = [],
     retries: int = 3,
     yolo_mode: bool | list[str] | None = None,
-) -> "Agent":
+) -> "Agent[None, Any]":
     """Creates a new Agent instance with configured tools and servers."""
-    from pydantic_ai import Agent, Tool
+    from pydantic_ai import Agent, RunContext, Tool
     from pydantic_ai.tools import GenerateToolJsonSchema
+    from pydantic_ai.toolsets import ToolsetTool, WrapperToolset
+
+    @dataclass
+    class ConfirmationWrapperToolset(WrapperToolset):
+        ctx: AnyContext
+        yolo_mode: bool | list[str]
+
+        async def call_tool(
+            self, name: str, tool_args: dict, ctx: RunContext, tool: ToolsetTool[None]
+        ) -> Any:
+            # The `tool` object is passed in. Use it for inspection.
+            # Define a temporary function that performs the actual tool call.
+            async def execute_delegated_tool_call(**params):
+                # Pass all arguments down the chain.
+                return await self.wrapped.call_tool(name, tool_args, ctx, tool)
+
+            # For the confirmation UI, make our temporary function look like the real one.
+            try:
+                execute_delegated_tool_call.__name__ = name
+                execute_delegated_tool_call.__doc__ = tool.function.__doc__
+                execute_delegated_tool_call.__signature__ = inspect.signature(
+                    tool.function
+                )
+            except (AttributeError, TypeError):
+                pass  # Ignore if we can't inspect the original function
+            # Use the existing wrap_func to get the confirmation logic
+            wrapped_executor = wrap_func(
+                execute_delegated_tool_call, self.ctx, self.yolo_mode
+            )
+            # Call the wrapped executor. This will trigger the confirmation prompt.
+            return await wrapped_executor(**tool_args)
 
     if yolo_mode is None:
         yolo_mode = False
@@ -62,12 +97,18 @@ def create_agent_instance(
         else:
             # Turn function into tool
             tool_list.append(wrap_tool(tool_or_callable, ctx, yolo_mode))
+    # Wrap toolsets
+    wrapped_toolsets = [
+        ConfirmationWrapperToolset(wrapped=toolset, ctx=ctx, yolo_mode=yolo_mode)
+        for toolset in toolsets
+    ]
     # Return Agent
-    return Agent(
+    return Agent[None, Any](
         model=model,
+        output_type=output_type,
         system_prompt=system_prompt,
         tools=tool_list,
-        toolsets=toolsets,
+        toolsets=wrapped_toolsets,
         model_settings=model_settings,
         retries=retries,
     )
@@ -75,56 +116,65 @@ def create_agent_instance(
 
 def get_agent(
     ctx: AnyContext,
-    agent_attr: "Agent | Callable[[AnySharedContext], Agent] | None",
     model: "str | Model",
-    system_prompt: str,
-    model_settings: "ModelSettings | None",
+    output_type: "OutputSpec[OutputDataT]" = str,
+    system_prompt: str = "",
+    model_settings: "ModelSettings | None" = None,
     tools_attr: (
         "list[ToolOrCallable] | Callable[[AnySharedContext], list[ToolOrCallable]]"
-    ),
-    additional_tools: "list[ToolOrCallable]",
-    toolsets_attr: "list[AbstractToolset[Agent]] | Callable[[AnySharedContext], list[AbstractToolset[Agent]]]",  # noqa
-    additional_toolsets: "list[AbstractToolset[Agent]]",
+    ) = [],
+    additional_tools: "list[ToolOrCallable]" = [],
+    toolsets_attr: "list[AbstractToolset[None] | str] | Callable[[AnySharedContext], list[AbstractToolset[None] | str]]" = [],  # noqa
+    additional_toolsets: "list[AbstractToolset[None] | str]" = [],
     retries: int = 3,
     yolo_mode: bool | list[str] | None = None,
 ) -> "Agent":
     """Retrieves the configured Agent instance or creates one if necessary."""
-    from pydantic_ai import Agent
-
-    # Render agent instance and return if agent_attr is already an agent
-    if isinstance(agent_attr, Agent):
-        return agent_attr
-    if callable(agent_attr):
-        agent_instance = agent_attr(ctx)
-        if not isinstance(agent_instance, Agent):
-            err_msg = (
-                "Callable agent factory did not return an Agent instance, "
-                f"got: {type(agent_instance)}"
-            )
-            raise TypeError(err_msg)
-        return agent_instance
     # Get tools for agent
     tools = list(tools_attr(ctx) if callable(tools_attr) else tools_attr)
     tools.extend(additional_tools)
     # Get Toolsets for agent
-    tool_sets = list(toolsets_attr(ctx) if callable(toolsets_attr) else toolsets_attr)
-    tool_sets.extend(additional_toolsets)
+    toolset_or_str_list = list(
+        toolsets_attr(ctx) if callable(toolsets_attr) else toolsets_attr
+    )
+    toolset_or_str_list.extend(additional_toolsets)
+    toolsets = _render_toolset_or_str_list(ctx, toolset_or_str_list)
     # If no agent provided, create one using the configuration
     return create_agent_instance(
         ctx=ctx,
         model=model,
+        output_type=output_type,
         system_prompt=system_prompt,
         tools=tools,
-        toolsets=tool_sets,
+        toolsets=toolsets,
         model_settings=model_settings,
         retries=retries,
         yolo_mode=yolo_mode,
     )
 
 
+def _render_toolset_or_str_list(
+    ctx: AnyContext, toolset_or_str_list: list["AbstractToolset[None] | str"]
+) -> list["AbstractToolset[None]"]:
+    from pydantic_ai.mcp import load_mcp_servers
+
+    toolsets = []
+    for toolset_or_str in toolset_or_str_list:
+        if isinstance(toolset_or_str, str):
+            try:
+                servers = load_mcp_servers(toolset_or_str)
+                for server in servers:
+                    toolsets.append(server)
+            except Exception as e:
+                ctx.log_error(f"Invalid MCP Config {toolset_or_str}: {e}")
+            continue
+        toolsets.append(toolset_or_str)
+    return toolsets
+
+
 async def run_agent_iteration(
     ctx: AnyContext,
-    agent: "Agent",
+    agent: "Agent[None, Any]",
     user_prompt: str,
     attachments: "list[UserContent] | None" = None,
     history_list: ListOfDict | None = None,
@@ -199,7 +249,6 @@ async def _run_single_agent_iteration(
         ) as agent_run:
             async for node in agent_run:
                 # Each node represents a step in the agent's execution
-                # Reference: https://ai.pydantic.dev/agents/#streaming
                 try:
                     await print_node(
                         _get_plain_printer(ctx), agent_run, node, log_indent_level
@@ -217,7 +266,7 @@ async def _run_single_agent_iteration(
 
 
 def _create_print_throttle_notif(ctx: AnyContext) -> Callable[[], None]:
-    def _print_throttle_notif(ctx: AnyContext):
+    def _print_throttle_notif():
         ctx.print(stylize_faint("  ⌛>> Request Throttled"), plain=True)
 
     return _print_throttle_notif
