@@ -2,11 +2,12 @@ import os
 import platform
 import re
 from datetime import datetime, timezone
-from typing import TYPE_CHECKING, Callable
+from typing import TYPE_CHECKING, Any, Callable
 
 from zrb.attr.type import StrAttr, StrListAttr
-from zrb.config.llm_config import llm_config as llm_config
+from zrb.config.llm_config import llm_config
 from zrb.config.llm_context.config import llm_context_config
+from zrb.config.llm_rate_limitter import llm_rate_limitter
 from zrb.context.any_context import AnyContext
 from zrb.context.any_shared_context import AnySharedContext
 from zrb.task.llm.conversation_history_model import ConversationHistory
@@ -88,6 +89,16 @@ def get_modes(
     return llm_config.default_modes or []
 
 
+def get_project_context_prompt() -> str:
+    context_dict = llm_context_config.get_contexts()
+    context_prompts = []
+    for context_path, context in context_dict.items():
+        context_prompts.append(
+            make_prompt_section(header=f"Context at `{context_path}`", content=context)
+        )
+    return "\n".join(context_prompts)
+
+
 def get_workflow_prompt(
     ctx: AnyContext,
     modes_attr: StrListAttr | None,
@@ -163,7 +174,43 @@ def get_system_and_user_prompt(
     render_modes: bool = False,
     conversation_history: ConversationHistory | None = None,
 ) -> tuple[str, str]:
-    """Combines persona, base system prompt, and special instructions."""
+    if conversation_history is None:
+        conversation_history = ConversationHistory()
+    modified_user_message, user_message_context = _extract_user_prompt_components(
+        user_message
+    )
+    new_system_prompt = _construct_system_prompt(
+        ctx=ctx,
+        persona_attr=persona_attr,
+        render_persona=render_persona,
+        system_prompt_attr=system_prompt_attr,
+        render_system_prompt=render_system_prompt,
+        special_instruction_prompt_attr=special_instruction_prompt_attr,
+        render_special_instruction_prompt=render_special_instruction_prompt,
+        modes_attr=modes_attr,
+        render_modes=render_modes,
+        conversation_history=conversation_history,
+    )
+    user_message_context["Token Counter"] = _get_token_usage_info(
+        new_system_prompt, user_message, conversation_history
+    )
+    return new_system_prompt, _contruct_user_message_prompt(
+        modified_user_message, user_message_context
+    )
+
+
+def _construct_system_prompt(
+    ctx: AnyContext,
+    persona_attr: StrAttr | None = None,
+    render_persona: bool = False,
+    system_prompt_attr: StrAttr | None = None,
+    render_system_prompt: bool = False,
+    special_instruction_prompt_attr: StrAttr | None = None,
+    render_special_instruction_prompt: bool = False,
+    modes_attr: StrListAttr | None = None,
+    render_modes: bool = False,
+    conversation_history: ConversationHistory | None = None,
+) -> str:
     persona = get_persona(ctx, persona_attr, render_persona)
     base_system_prompt = get_base_system_prompt(
         ctx, system_prompt_attr, render_system_prompt
@@ -172,10 +219,12 @@ def get_system_and_user_prompt(
         ctx, special_instruction_prompt_attr, render_special_instruction_prompt
     )
     workflow_prompt = get_workflow_prompt(ctx, modes_attr, render_modes)
+    project_context_prompt = get_project_context_prompt()
     if conversation_history is None:
         conversation_history = ConversationHistory()
-    conversation_context, new_user_message = extract_conversation_context(user_message)
-    new_system_prompt = "\n".join(
+    current_directory = os.getcwd()
+    directory_tree = _generate_directory_tree(current_directory, max_depth=2)
+    return "\n".join(
         [
             make_prompt_section("Persona", persona),
             make_prompt_section("System Prompt", base_system_prompt),
@@ -215,13 +264,100 @@ def get_system_and_user_prompt(
                     ]
                 ),
             ),
-            make_prompt_section("Conversation Context", conversation_context),
+            make_prompt_section("Project Context", project_context_prompt),
+            make_prompt_section(
+                "Conversation Environment",
+                "\n".join(
+                    [
+                        make_prompt_section("Current OS", platform.system()),
+                        make_prompt_section("OS Version", platform.version()),
+                        make_prompt_section(
+                            "Python Version", platform.python_version()
+                        ),
+                        make_prompt_section(
+                            "Directory Tree (depth=2)", directory_tree, as_code=True
+                        ),
+                    ]
+                ),
+            ),
         ]
     )
-    return new_system_prompt, new_user_message
 
 
-def extract_conversation_context(user_message: str) -> tuple[str, str]:
+def _contruct_user_message_prompt(
+    modified_user_message: str, user_message_context: dict[str, str]
+) -> str:
+    return "\n".join(
+        [
+            make_prompt_section("User Message", modified_user_message),
+            make_prompt_section(
+                "Context",
+                "\n".join(
+                    [
+                        make_prompt_section(key, val)
+                        for key, val in user_message_context.items()
+                    ]
+                ),
+            ),
+        ]
+    )
+
+
+def _generate_directory_tree(
+    dir_path: str,
+    max_depth: int = 2,
+    max_children: int = 10,
+) -> str:
+    """
+    Generates a string representation of a directory tree, pure Python.
+    """
+    tree_lines = []
+
+    def recurse(path: str, depth: int, prefix: str):
+        if depth > max_depth:
+            return
+        try:
+            entries = sorted(
+                [e for e in os.scandir(path)],
+                key=lambda e: e.name,
+            )
+        except FileNotFoundError:
+            return
+
+        for i, entry in enumerate(entries):
+            if i >= max_children:
+                tree_lines.append(f"{prefix}└─... (more)")
+                break
+            is_last = i == len(entries) - 1
+            connector = "└─" if is_last else "├─"
+            tree_lines.append(f"{prefix}{connector}{entry.name}")
+            if entry.is_dir():
+                new_prefix = prefix + ("  " if is_last else "│ ")
+                recurse(entry.path, depth + 1, new_prefix)
+
+    tree_lines.append(os.path.basename(dir_path))
+    recurse(dir_path, 1, "")
+    return "\n".join(tree_lines)
+
+
+def _get_token_usage_info(
+    system_prompt: str, user_message: str, conversation_history: ConversationHistory
+) -> str:
+    system_prompt_token_count = llm_rate_limitter.count_token(system_prompt)
+    user_message_token_count = llm_rate_limitter.count_token(user_message)
+    conversation_history_token_count = llm_rate_limitter.count_token(
+        conversation_history.history
+    )
+    total_token_count = (
+        system_prompt_token_count
+        + user_message_token_count
+        + conversation_history_token_count
+    )
+    max_token = llm_rate_limitter.max_tokens_per_request
+    return f"{total_token_count} tokens of {max_token} tokens"
+
+
+def _extract_user_prompt_components(user_message: str) -> tuple[str, dict[str, Any]]:
     modified_user_message = user_message
     # Match “@” + any non-space/comma sequence that contains at least one “/”
     pattern = r"(?<!\w)@(?=[^,\s]*\/)([^,\?\!\s]+)"
@@ -246,37 +382,18 @@ def extract_conversation_context(user_message: str) -> tuple[str, str]:
             apendixes.append(
                 make_prompt_section(
                     f"{placeholder} ({ref_type} path: `{resource_path}`)",
-                    content,
+                    "\n".join(content) if isinstance(content, list) else content,
                     as_code=True,
                 )
             )
-    conversation_context = "\n".join(
-        [
-            make_prompt_section("Current OS", platform.system()),
-            make_prompt_section("OS Version", platform.version()),
-            make_prompt_section("Python Version", platform.python_version()),
-        ]
-    )
-    iso_date = datetime.now(timezone.utc).astimezone().isoformat()
     current_directory = os.getcwd()
-    modified_user_message = "\n".join(
-        [
-            make_prompt_section("User Message", modified_user_message),
-            make_prompt_section(
-                "Context",
-                "\n".join(
-                    [
-                        make_prompt_section(
-                            "Current working directory", current_directory
-                        ),
-                        make_prompt_section("Current time", iso_date),
-                        make_prompt_section("Apendixes", "\n".join(apendixes)),
-                    ]
-                ),
-            ),
-        ]
-    )
-    return conversation_context, modified_user_message
+    iso_date = datetime.now(timezone.utc).astimezone().isoformat()
+    user_message_context = {
+        "Current Working Directory": current_directory,
+        "Current Time": iso_date,
+        "Apendixes": "\n".join(apendixes),
+    }
+    return modified_user_message, user_message_context
 
 
 def get_user_message(
