@@ -1,10 +1,13 @@
 import functools
 import inspect
+import json
 import traceback
 import typing
 from collections.abc import Callable
 from typing import TYPE_CHECKING, Any
 
+from zrb.config.config import CFG
+from zrb.config.llm_rate_limitter import llm_rate_limitter
 from zrb.context.any_context import AnyContext
 from zrb.task.llm.error import ToolExecutionError
 from zrb.util.callable import get_callable_name
@@ -15,6 +18,7 @@ from zrb.util.cli.style import (
     stylize_green,
     stylize_yellow,
 )
+from zrb.util.cli.text import edit_text
 from zrb.util.run import run_async
 from zrb.util.string.conversion import to_boolean
 
@@ -39,7 +43,6 @@ def wrap_tool(func: Callable, ctx: AnyContext, yolo_mode: bool | list[str]) -> "
 def wrap_func(func: Callable, ctx: AnyContext, yolo_mode: bool | list[str]) -> Callable:
     original_sig = inspect.signature(func)
     needs_any_context_for_injection = _has_context_parameter(original_sig, AnyContext)
-    takes_no_args = len(original_sig.parameters) == 0
     # Pass individual flags to the wrapper creator
     wrapper = _create_wrapper(
         func=func,
@@ -48,7 +51,7 @@ def wrap_func(func: Callable, ctx: AnyContext, yolo_mode: bool | list[str]) -> C
         needs_any_context_for_injection=needs_any_context_for_injection,
         yolo_mode=yolo_mode,
     )
-    _adjust_signature(wrapper, original_sig, takes_no_args)
+    _adjust_signature(wrapper, original_sig)
     return wrapper
 
 
@@ -108,20 +111,28 @@ def _create_wrapper(
             # Inject the captured ctx into kwargs. This will overwrite if the LLM
             # somehow provided it.
             kwargs[any_context_param_name] = ctx
-        # If the dummy argument was added for schema generation and is present in kwargs,
-        # remove it before calling the original function, unless the original function
-        # actually expects a parameter named '_dummy'.
-        if "_dummy" in kwargs and "_dummy" not in original_sig.parameters:
-            del kwargs["_dummy"]
         try:
+            has_ever_edited = False
             if not ctx.is_web_mode and ctx.is_tty:
                 if (
                     isinstance(yolo_mode, list) and func.__name__ not in yolo_mode
                 ) or not yolo_mode:
-                    approval, reason = await _ask_for_approval(ctx, func, args, kwargs)
+                    approval, reason, has_ever_edited = await _handle_user_response(
+                        ctx, func, args, kwargs
+                    )
                     if not approval:
-                        raise ToolExecutionCancelled(f"User disapproving: {reason}")
-            return await run_async(func(*args, **kwargs))
+                        raise ToolExecutionCancelled(
+                            f"Tool execution cancelled. User disapproving: {reason}"
+                        )
+            result = await run_async(func(*args, **kwargs))
+            _check_tool_call_result_limit(result)
+            if has_ever_edited:
+                return {
+                    "tool_call_result": result,
+                    "new_tool_parameters": kwargs,
+                    "message": "User correction: Tool was called with user's parameters",
+                }
+            return result
         except KeyboardInterrupt as e:
             raise e
         except Exception as e:
@@ -136,54 +147,107 @@ def _create_wrapper(
     return wrapper
 
 
-async def _ask_for_approval(
-    ctx: AnyContext, func: Callable, args: list[Any], kwargs: dict[str, Any]
-) -> tuple[bool, str]:
-    func_call_str = _get_func_call_str(func, args, kwargs)
-    complete_confirmation_message = "\n".join(
-        [
-            f"\n🎰 >> {func_call_str}",
-            _get_detail_func_param(args, kwargs),
-            f"🎰  >> {_get_run_func_confirmation(func)}",
-        ]
-    )
+def _check_tool_call_result_limit(result: Any):
+    if (
+        llm_rate_limitter.count_token(result)
+        > llm_rate_limitter.max_tokens_per_tool_call_result
+    ):
+        raise ValueError("Result value is too large, please adjust the parameter")
+
+
+async def _handle_user_response(
+    ctx: AnyContext,
+    func: Callable,
+    args: list[Any] | tuple[Any],
+    kwargs: dict[str, Any],
+) -> tuple[bool, str, bool]:
+    has_ever_edited = False
     while True:
+        func_call_str = _get_func_call_str(func, args, kwargs)
+        complete_confirmation_message = "\n".join(
+            [
+                f"\n🎰 >> {func_call_str}",
+                _get_detail_func_param(args, kwargs),
+                f"🎰  >> {_get_run_func_confirmation(func)}",
+            ]
+        )
         ctx.print(complete_confirmation_message, plain=True)
-        user_input = await _read_line()
+        user_response = await _read_line()
         ctx.print("", plain=True)
-        user_responses = [val.strip() for val in user_input.split(",", maxsplit=1)]
-        while len(user_responses) < 2:
-            user_responses.append("")
-        approval_str, reason = user_responses
-        try:
-            approved = True if approval_str.strip() == "" else to_boolean(approval_str)
-            if not approved and reason == "":
-                ctx.print(
-                    stylize_error(
-                        f"You must specify rejection reason (i.e., No, <why>) for {func_call_str}"  # noqa
-                    ),
-                    plain=True,
-                )
-                continue
-            return approved, reason
-        except Exception:
+        new_kwargs, is_edited = _get_edited_kwargs(ctx, user_response, kwargs)
+        if is_edited:
+            kwargs = new_kwargs
+            has_ever_edited = True
+            continue
+        approval_and_reason = _get_user_approval_and_reason(
+            ctx, user_response, func_call_str
+        )
+        if approval_and_reason is None:
+            continue
+        approval, reason = approval_and_reason
+        return approval, reason, has_ever_edited
+
+
+def _get_edited_kwargs(
+    cx: AnyContext, user_response: str, kwargs: dict[str, Any]
+) -> tuple[dict[str, Any], bool]:
+    user_edit_responses = [val for val in user_response.split(" ", maxsplit=2)]
+    if len(user_edit_responses) >= 1 and user_edit_responses[0].lower() != "edit":
+        return kwargs, False
+    while len(user_edit_responses) < 3:
+        user_edit_responses.append("")
+    key, val_str = user_edit_responses[1:]
+    if key not in kwargs:
+        return kwargs, True
+    is_str_param = isinstance(kwargs[key], str)
+    if val_str == "":
+        val_str = edit_text(
+            prompt_message=f"// {key}",
+            value=_get_val_str(kwargs[key]),
+            editor=CFG.DEFAULT_EDITOR,
+        )
+    kwargs[key] = val_str if is_str_param else json.loads(val_str)
+    return kwargs, True
+
+
+def _get_user_approval_and_reason(
+    ctx: AnyContext, user_response: str, func_call_str: str
+) -> tuple[bool, str] | None:
+    user_approval_responses = [
+        val.strip() for val in user_response.split(",", maxsplit=1)
+    ]
+    while len(user_approval_responses) < 2:
+        user_approval_responses.append("")
+    approval_str, reason = user_approval_responses
+    try:
+        approved = True if approval_str.strip() == "" else to_boolean(approval_str)
+        if not approved and reason == "":
             ctx.print(
                 stylize_error(
-                    f"Invalid approval value for {func_call_str}: {approval_str}"
+                    f"You must specify rejection reason (i.e., No, <why>) for {func_call_str}"  # noqa
                 ),
                 plain=True,
             )
-            continue
+            return None
+        return approved, reason
+    except Exception:
+        ctx.print(
+            stylize_error(
+                f"Invalid approval value for {func_call_str}: {approval_str}"
+            ),
+            plain=True,
+        )
+        return None
 
 
 def _get_run_func_confirmation(func: Callable) -> str:
     func_name = get_callable_name(func)
     return render_markdown(
-        f"Allow to run `{func_name}`? (✅ `Yes` | ⛔ `No, <reason>`)"
+        f"Allow to run `{func_name}`? (✅ `Yes` | ⛔ `No, <reason>` | 📝 `Edit <param> <value>`)"
     ).strip()
 
 
-def _get_detail_func_param(args: list[Any], kwargs: dict[str, Any]) -> str:
+def _get_detail_func_param(args: list[Any] | tuple[Any], kwargs: dict[str, Any]) -> str:
     markdown = "\n".join(
         [_get_func_param_item(key, val) for key, val in kwargs.items()]
     )
@@ -191,19 +255,29 @@ def _get_detail_func_param(args: list[Any], kwargs: dict[str, Any]) -> str:
 
 
 def _get_func_param_item(key: str, val: Any) -> str:
-    upper_key = key.upper()
-    val_str = f"{val}"
+    val_str = _get_val_str(val)
     val_parts = val_str.split("\n")
     if len(val_parts) == 1:
-        return f"- {upper_key} `{val}`"
-    lines = [f"- {upper_key}", "  ```"]
+        return f"- {key} `{val}`"
+    lines = [f"- {key}", "  ```"]
     for val_part in val_parts:
         lines.append(f"  {val_part}")
     lines.append("  ```")
     return "\n".join(lines)
 
 
-def _get_func_call_str(func: Callable, args: list[Any], kwargs: dict[str, Any]) -> str:
+def _get_val_str(val: Any) -> str:
+    if isinstance(val, str):
+        return val
+    try:
+        return json.dumps(val, indent=4)
+    except Exception:
+        return f"{val}"
+
+
+def _get_func_call_str(
+    func: Callable, args: list[Any] | tuple[Any], kwargs: dict[str, Any]
+) -> str:
     func_name = get_callable_name(func)
     normalized_args = [stylize_green(_truncate_arg(arg)) for arg in args]
     normalized_kwargs = []
@@ -230,9 +304,7 @@ async def _read_line():
     return await reader.prompt_async()
 
 
-def _adjust_signature(
-    wrapper: Callable, original_sig: inspect.Signature, takes_no_args: bool
-):
+def _adjust_signature(wrapper: Callable, original_sig: inspect.Signature):
     """Adjusts the wrapper function's signature for schema generation."""
     # The wrapper's signature should represent the arguments the *LLM* needs to provide.
     # The LLM does not provide RunContext (pydantic-ai injects it) or AnyContext
@@ -247,22 +319,4 @@ def _adjust_signature(
         if not _is_annotated_with_context(param.annotation, RunContext)
         and not _is_annotated_with_context(param.annotation, AnyContext)
     ]
-
-    # If after removing context parameters, there are no parameters left,
-    # and the original function took no args, keep the dummy.
-    # If after removing context parameters, there are no parameters left,
-    # but the original function *did* take args (only context), then the schema
-    # should have no parameters.
-    if not params_for_schema and takes_no_args:
-        # Keep the dummy if the original function truly had no parameters
-        new_sig = inspect.Signature(
-            parameters=[
-                inspect.Parameter(
-                    "_dummy", inspect.Parameter.POSITIONAL_OR_KEYWORD, default=None
-                )
-            ]
-        )
-    else:
-        new_sig = inspect.Signature(parameters=params_for_schema)
-
-    wrapper.__signature__ = new_sig
+    wrapper.__signature__ = inspect.Signature(parameters=params_for_schema)
